@@ -1,5 +1,6 @@
+# Controls the character string content within a Tk entry widget.
+
 CheckEntry <- function (ent.typ, ent.str="") {
-  # Controls the character string content within a Tk entry widget.
 
   if (ent.str == "")
     return("")
@@ -7,10 +8,14 @@ CheckEntry <- function (ent.typ, ent.str="") {
   chr <- unlist(strsplit(ent.str, split=""))
 
   if (ent.typ == "numeric") {
-    accept.vals <- c(as.character(0:9), ".", "-")
+    accept.vals <- c(as.character(0:9), "-", "e", "E", ".")
+  } else if (ent.typ %in% "integer") {
+    accept.vals <- c(as.character(0:9), "-", "e", "E")
+  } else if (ent.typ == "logical") {
+    accept.vals <- c("T", "R", "U", "E", "F", "A", "L", "S")
   } else if (ent.typ == "second") {
     accept.vals <- c(as.character(0:9), ".")
-  } else if (ent.typ %in% c("integer", "hour", "minute")) {
+  } else if (ent.typ %in% c("hour", "minute")) {
     accept.vals <- c(as.character(0:9))
   } else if (ent.typ == "date") {
     accept.vals <- c("a", "A", "b", "c", "C", "d", "D", "e", "E", "F", "g",
@@ -42,5 +47,5 @@ CheckEntry <- function (ent.typ, ent.str="") {
     }
   }
 
-  rtn
+  return(rtn)
 }
