@@ -1,9 +1,10 @@
-EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL, 
-                         value.class=NULL, win.title="Edit Function",  
-                         parent=NULL) {
-  # A GUI for defining a function in the R language with a focus on table data.
+# A GUI for defining a function in the R language with a focus on table data.
 
-  # Additional functions (subroutines)
+EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
+                         value.class=NULL, win.title="Edit Function",
+                         parent=NULL) {
+
+  ## Additional functions (subroutines)
 
   # Save function
   SaveFunction <- function() {
@@ -12,12 +13,12 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
       rtn <<- list(fun="")
     } else {
       fun <- txt
-      pattern <- paste("\"", ids, "\"", sep="")
-      replacement <- paste("DATA[[", 1:length(ids), "]]", sep="")
+      pattern <- paste0("\"", ids, "\"")
+      replacement <- paste0("DATA[[", 1:length(ids), "]]")
       for (i in seq(along=ids))
         fun <- gsub(pattern[i], replacement[i], fun, fixed=TRUE)
-      fun <- paste("function(DATA) {", fun, "}", sep="")
-      
+      fun <- paste0("function(DATA) {", fun, "}")
+
       fun <- try(parse(text=fun), silent=TRUE)
       if (inherits(fun, "try-error")) {
         msg <- "There's a problem with function syntax, try revising."
@@ -32,27 +33,28 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
                      type="ok", parent=tt)
         return()
       }
-      
+
       if (!is.null(value.length) && length(obj) != value.length) {
-        msg <- paste("Evaluated function must be of length ", value.length, 
-                     ", try revising.", sep="")
-        dtl <- paste("Resulting object is currently of length ", length(obj), 
-                     ".", sep="")
+        msg <- paste0("Evaluated function must be of length ", value.length,
+                      ", try revising.")
+        dtl <- paste0("Resulting vector is currently of length ", length(obj),
+                      ".")
         tkmessageBox(icon="error", message=msg, detail=dtl, title="Error",
                      type="ok", parent=tt)
         return()
       }
-      
+
       if (!is.null(value.class) && !inherits(obj, value.class)) {
-        msg <- paste("A query must result in an object of class \"", value.class, 
-                     "\". The evaluated function is an object of class \"", 
-                     class(obj), "\", please revise.", sep="")
-        tkmessageBox(icon="error", message=msg, title="Error", type="ok", 
+        msg <- paste0("A query must result in a vector of class \"",
+                      value.class,
+                      "\". The evaluated function is a vector of class \"",
+                      class(obj), "\", please revise.")
+        tkmessageBox(icon="error", message=msg, title="Error", type="ok",
                      parent=tt)
         return()
       }
-      
-      rtn <<- list(fun=txt, class=class(obj)[1], summary=SummarizeData(obj), 
+
+      rtn <<- list(fun=txt, class=class(obj)[1], summary=SummarizeVariable(obj),
                    sample=na.omit(obj)[1])
     }
     tclvalue(tt.done.var) <- 1
@@ -70,9 +72,9 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
       tcl("lappend", variable.var, show.ids[i])
 
     tkselection.clear(frame1.lst.2.1, 0, "end")
-    
+
     tclvalue(value.var) <- ""
-    
+
     tkconfigure(frame1.but.5.1, state="disabled")
     tkfocus(frame2.txt.2.1)
   }
@@ -117,7 +119,7 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
     if (length(idx) == 0)
       return()
     id <- as.character(tkget(frame1.lst.2.1, idx, idx))
-    txt <- paste("\"", id, "\"", sep="")
+    txt <- paste0("\"", id, "\"")
     InsertString(txt)
   }
 
@@ -132,11 +134,11 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
   # Text edit functions
   EditUndo <- function() {
     tkfocus(frame2.txt.2.1)
-    tcl(frame2.txt.2.1, "edit", "undo")
+    try(tcl(frame2.txt.2.1, "edit", "undo"), silent=TRUE)
   }
   EditRedo <- function() {
     tkfocus(frame2.txt.2.1)
-    tcl(frame2.txt.2.1, "edit", "redo")
+    try(tcl(frame2.txt.2.1, "edit", "redo"), silent=TRUE)
   }
   EditCut <- function() {
     tkfocus(frame2.txt.2.1)
@@ -154,13 +156,13 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
     tkfocus(frame2.txt.2.1)
     tktag.add(frame2.txt.2.1, "sel", "1.0", "end")
   }
-  
-  # Clear all
-  ClearAll <- function() {
+
+  # Clear console
+  ClearConsole <- function() {
     tcl(frame2.txt.2.1, "delete", "1.0", "end")
     tkfocus(frame2.txt.2.1)
   }
-  
+
   # Show unique values
   ShowUniqueValues <- function() {
     idx <- as.integer(tkcurselection(frame1.lst.2.1))
@@ -168,9 +170,9 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
       return()
     id <- as.character(tkget(frame1.lst.2.1, idx, idx))
     idx <- which(vapply(cols, function(i) i$id, "") == id)
-    
+
     var.fmt <- cols[[idx]]$format
-    
+
     tkconfigure(tt, cursor="watch")
     var.vals <- unique(EvalFunction(cols[[idx]]$fun, cols))
     var.class <- cols[[idx]]$class
@@ -187,9 +189,9 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
         return()
       }
     }
-    
+
     var.vals <- sort(var.vals, na.last=TRUE)
-    if (is.null(var.fmt)) {
+    if (var.fmt == "") {
       var.vals.txt <- format(var.vals)
     } else if (var.class == "POSIXct") {
       var.vals.txt <- format(var.vals, format=var.fmt)
@@ -199,7 +201,7 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
         var.vals.txt <- format(var.vals)
     }
     var.vals.txt <- gsub("^\\s+|\\s+$", "", var.vals.txt)
-    
+
     tclvalue(value.var) <- ""
     for (i in seq(along=var.vals.txt))
       tcl("lappend", value.var, var.vals.txt[i])
@@ -208,7 +210,7 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
     tkfocus(frame2.txt.2.1)
     tkconfigure(tt, cursor="arrow")
   }
-  
+
   # Change variable selection
   ChangeVar <- function() {
     tclvalue(value.var) <- ""
@@ -217,7 +219,7 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
       return()
     tkconfigure(frame1.but.5.1, state="normal")
   }
-  
+
   # Insert value into text box
   InsertValue <- function() {
     idx <- as.integer(tkcurselection(frame1.lst.2.1))
@@ -234,28 +236,43 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
     if (var.class == "factor" && is.na(suppressWarnings(as.numeric(val))))
       var.class <- "character"
     if (var.class == "POSIXct") {
-      txt <- paste("as.POSIXct(\"", val, "\", format = \"", var.fmt, "\")", 
-                   sep="")
-    } else if (var.class == "integer" && val != "NA") {
-      txt <- paste(val, "L", sep="")
-    } else if (var.class == "character" && val != "NA") {
-      txt <- paste("\"", val, "\"", sep="")
+      txt <- paste0("as.POSIXct(\"", val, "\", format = \"", var.fmt, "\")")
+    } else if (var.class == "integer" &&
+               !val %in% c("NA", "NaN", "Inf", "-Inf")) {
+      txt <- paste0(val, "L")
+    } else if (var.class == "character" &&
+               !val %in% c("NA", "NaN", "Inf", "-Inf")) {
+      txt <- paste0("\"", val, "\"")
     } else {
       txt <- val
     }
     InsertString(txt)
   }
 
+  # Insert trigonometric function
+  InsertTrigFunction <- function(fun) {
+    is.inverse <- as.logical(as.integer(tclvalue(inverse.var)))
+    is.hyperbolic <- as.logical(as.integer(tclvalue(hyperbolic.var)))
+    angles <- as.character(tclvalue(angles.var))
+    if (is.inverse)
+      fun <- paste0("a", fun)
+    if (is.hyperbolic)
+      fun <- paste0(fun, "h")
+    if (angles == "deg")
+      InsertString(paste0(fun, "(<variable> * pi / 180)"))
+    else
+      InsertString(paste0(fun, "(<variable>)"))
+  }
+
+  ## Main program
 
-  # Main program
-  
   if (is.null(index)) {
     old.fun <- fun
   } else {
     old.fun <- cols[[as.integer(index)]]$fun
   }
   rtn <- NULL
-  
+
   ids <- vapply(cols, function(i) i$id, "")
   cls <- vapply(cols, function(i) i$class, "")
   if (!is.null(index)) {
@@ -264,6 +281,9 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
     cls <- cls[-index]
   }
 
+  if (!is.null(value.length))
+    value.length <- as.integer(value.length)
+
   # Class types
   classes <- c("numeric", "integer", "POSIXct", "logical",
                "character", "factor")
@@ -272,8 +292,11 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
   # Assign variables linked to Tk widgets
   variable.var <- tclVar()
   for (i in seq(along=ids))
-    tcl("lappend", variable.var, ids[i]) # must be unique
+    tcl("lappend", variable.var, ids[i])  # must be unique
   value.var <- tclVar()
+  inverse.var <- tclVar(0)
+  hyperbolic.var <- tclVar(0)
+  angles.var <- tclVar("rad")
   tt.done.var <- tclVar(0)
 
   # Open GUI
@@ -284,8 +307,8 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
   if (!is.null(parent)) {
     tkwm.transient(tt, parent)
     geo <- unlist(strsplit(as.character(tkwm.geometry(parent)), "\\+"))
-    tkwm.geometry(tt, paste("+", as.integer(geo[2]) + 25,
-                            "+", as.integer(geo[3]) + 25, sep=""))
+    tkwm.geometry(tt, paste0("+", as.integer(geo[2]) + 25,
+                             "+", as.integer(geo[3]) + 25))
   }
   tktitle(tt) <- win.title
 
@@ -296,66 +319,110 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
 
   menu.edit <- tkmenu(tt, tearoff=0)
   tkadd(top.menu, "cascade", label="Edit", menu=menu.edit, underline=0)
-  tkadd(menu.edit, "command", label="Undo", accelerator="Ctrl+Z",
+  tkadd(menu.edit, "command", label="Undo", accelerator="Ctrl+z",
         command=EditUndo)
-  tkadd(menu.edit, "command", label="Redo", accelerator="Ctrl+Y",
+  tkadd(menu.edit, "command", label="Redo", accelerator="Ctrl+y",
         command=EditRedo)
   tkadd(menu.edit, "separator")
-  tkadd(menu.edit, "command", label="Cut", accelerator="Ctrl+X",
+  tkadd(menu.edit, "command", label="Cut", accelerator="Ctrl+x",
         command=EditCut)
-  tkadd(menu.edit, "command", label="Copy", accelerator="Ctrl+C",
+  tkadd(menu.edit, "command", label="Copy", accelerator="Ctrl+c",
         command=EditCopy)
-  tkadd(menu.edit, "command", label="Paste", accelerator="Ctrl+V",
+  tkadd(menu.edit, "command", label="Paste", accelerator="Ctrl+v",
         command=EditPaste)
   tkadd(menu.edit, "separator")
-  tkadd(menu.edit, "command", label="Select all", accelerator="Ctrl+A",
+  tkadd(menu.edit, "command", label="Select all", accelerator="Ctrl+a",
         command=EditSelectAll)
-  tkadd(menu.edit, "command", label="Clear all", 
-        command=ClearAll)
+  tkadd(menu.edit, "command", label="Clear console", accelerator="Ctrl+l",
+        command=ClearConsole)
 
   menu.class <- tkmenu(tt, tearoff=0)
   tkadd(top.menu, "cascade", label="Class", menu=menu.class, underline=0)
-  tkadd(menu.class, "command", label="As numeric",
-        command=function() InsertString("as.numeric(<variable>)"))
-  tkadd(menu.class, "command", label="As integer",
-        command=function() InsertString("as.integer(<variable>)"))
-  tkadd(menu.class, "command", label="As POSIXct",
-        command=function() {
-          InsertString("as.POSIXct(<variable>, format = \"<format>\")")
-        })
-  tkadd(menu.class, "command", label="As logical",
-        command=function()  InsertString("as.logical(<variable>)"))
+
   tkadd(menu.class, "command", label="As character",
         command=function() InsertString("as.character(<variable>)"))
   tkadd(menu.class, "command", label="As factor",
         command=function() InsertString("as.factor(<variable>)"))
-  
+  tkadd(menu.class, "command", label="As integer",
+        command=function() InsertString("as.integer(<variable>)"))
+  tkadd(menu.class, "command", label="As logical",
+        command=function()  InsertString("as.logical(<variable>)"))
+  tkadd(menu.class, "command", label="As numeric",
+        command=function() InsertString("as.numeric(<variable>)"))
+  tkadd(menu.class, "command", label="As POSIXct",
+        command=function() {
+          InsertString("as.POSIXct(<variable>, format = \"<format>\")")
+        })
+
   menu.math <- tkmenu(tt, tearoff=0)
   tkadd(top.menu, "cascade", label="Math", menu=menu.math, underline=0)
-  tkadd(menu.math, "command", label="Absolute value",
-        command=function() InsertString("abs(<variable>)"))
   tkadd(menu.math, "command", label="Square root",
         command=function() InsertString("sqrt(<variable>)"))
+  tkadd(menu.math, "command", label="Absolute value",
+        command=function() InsertString("abs(<variable>)"))
+  tkadd(menu.math, "separator")
+  tkadd(menu.math, "command", label="Floor",
+        command=function() InsertString("floor(<variable>)"))
+  tkadd(menu.math, "command", label="Ceiling",
+        command=function() InsertString("ceiling(<variable>)"))
+  tkadd(menu.math, "command", label="Truncation",
+        command=function() InsertString("trunc(<variable>)"))
+  menu.math.round <- tkmenu(tt, tearoff=0)
+  tkadd(menu.math.round, "command", label="Decimal places",
+        command=function() InsertString("round(<variable>, digits = 0)"))
+  tkadd(menu.math.round, "command", label="Significant digits",
+        command=function() InsertString("signif(<variable>, digits = 6)"))
+  tkadd(menu.math, "cascade", label="Round to", menu=menu.math.round)
+  tkadd(menu.math, "separator")
   tkadd(menu.math, "command", label="Exponential",
         command=function() InsertString("exp(<variable>)"))
-  tkadd(menu.math, "command", label="Logarithm",
+  menu.math.log <- tkmenu(tt, tearoff=0)
+  tkadd(menu.math.log, "command", label="Common (base 10)",
+        command=function() InsertString("log10(<variable>)"))
+  tkadd(menu.math.log, "command", label="Natural (base \u0065)",
         command=function() InsertString("log(<variable>, base = exp(1))"))
-  
+  tkadd(menu.math.log, "command", label="Binary (base 2)",
+        command=function() InsertString("log2(<variable>)"))
+  tkadd(menu.math, "cascade", label="Logarithm", menu=menu.math.log)
+  tkadd(menu.math, "separator")
+  tkadd(menu.math, "command", label="Sine",
+        command=function() InsertTrigFunction("sin"))
+  tkadd(menu.math, "command", label="Cosine",
+        command=function() InsertTrigFunction("cos"))
+  tkadd(menu.math, "command", label="Tangent",
+        command=function() InsertTrigFunction("tan"))
+  tkadd(menu.math, "separator")
+  menu.math.cum <- tkmenu(tt, tearoff=0)
+  tkadd(menu.math.cum, "command", label="Sum",
+        command=function() InsertString("cumsum(<variable>)"))
+  tkadd(menu.math.cum, "command", label="Product",
+        command=function() InsertString("cumprod(<variable>)"))
+  tkadd(menu.math, "cascade", label="Cumulative", menu=menu.math.cum)
+
   menu.summary <- tkmenu(tt, tearoff=0)
   tkadd(top.menu, "cascade", label="Summary", menu=menu.summary, underline=0)
+  tkadd(menu.summary, "command", label="Are all values true",
+        command=function() InsertString("all(<variable>, na.rm = FALSE)"))
+  tkadd(menu.summary, "command", label="Are any values true",
+        command=function() InsertString("any(<variable>, na.rm = FALSE)"))
+  tkadd(menu.summary, "separator")
   tkadd(menu.summary, "command", label="Sum",
         command=function() InsertString("sum(<variable>, na.rm = TRUE)"))
   tkadd(menu.summary, "command", label="Product",
         command=function() InsertString("prod(<variable>, na.rm = TRUE)"))
+  tkadd(menu.summary, "separator")
   tkadd(menu.summary, "command", label="Minimum",
         command=function() InsertString("min(<variable>, na.rm = TRUE)"))
   tkadd(menu.summary, "command", label="Maximum",
         command=function() InsertString("max(<variable>, na.rm = TRUE)"))
-  tkadd(menu.summary, "command", label="Median",
-        command=function() InsertString("median(<variable>, na.rm = TRUE)"))
+  tkadd(menu.summary, "separator")
   tkadd(menu.summary, "command", label="Mean",
         command=function() InsertString("mean(<variable>, na.rm = TRUE)"))
-        
+  tkadd(menu.summary, "command", label="Median",
+        command=function() InsertString("median(<variable>, na.rm = TRUE)"))
+  tkadd(menu.summary, "command", label="Standard deviation",
+        command=function() InsertString("sd(<variable>, na.rm = TRUE)"))
+
   menu.operator <- tkmenu(tt, tearoff=0)
   tkadd(top.menu, "cascade", label="Operator", menu=menu.operator, underline=0)
   tkadd(menu.operator, "command", label="And",
@@ -364,20 +431,51 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
         command=function() InsertString(" | "))
   tkadd(menu.operator, "command", label="Not",
         command=function() InsertString("!"))
+  tkadd(menu.operator, "separator")
+  tkadd(menu.operator, "command", label="In",
+        command=function() InsertString(" %in% "))
+  tkadd(menu.operator, "command", label="Match",
+        command=function() {
+          InsertString("match(<variable>, <values>, nomatch = NA)")
+        })
+  tkadd(menu.operator, "separator")
   tkadd(menu.operator, "command", label="Exponentiation",
         command=function() InsertString("<variable>^<power>"))
 
   menu.const <- tkmenu(tt, tearoff=0)
   tkadd(top.menu, "cascade", label="Constant", menu=menu.const, underline=0)
+  tkadd(menu.const, "command", label="\u03C0",
+        command=function() InsertString("pi"))
+  tkadd(menu.const, "command", label="\u0065",
+        command=function() InsertString("exp(1)"))
+  tkadd(menu.const, "separator")
   tkadd(menu.const, "command", label="True",
         command=function() InsertString("TRUE"))
   tkadd(menu.const, "command", label="False",
         command=function() InsertString("FALSE"))
+  tkadd(menu.const, "separator")
   tkadd(menu.const, "command", label="Not available",
         command=function() InsertString("NA"))
-  tkadd(menu.const, "command", label="Pi",
-        command=function() InsertString("pi"))
-  
+  tkadd(menu.const, "command", label="Not a number",
+        command=function() InsertString("NaN"))
+  tkadd(menu.const, "separator")
+  tkadd(menu.const, "command", label="Positive infinity",
+        command=function() InsertString("Inf"))
+  tkadd(menu.const, "command", label="Negative infinity",
+        command=function() InsertString("-Inf"))
+  tkadd(menu.const, "separator")
+  menu.const.is <- tkmenu(tt, tearoff=0)
+  tkadd(menu.const.is, "command", label="Not available",
+        command=function() InsertString("is.na(<variable>)"))
+  tkadd(menu.const.is, "command", label="Not a number",
+        command=function() InsertString("is.nan(<variable>)"))
+  tkadd(menu.const.is, "separator")
+  tkadd(menu.const.is, "command", label="Finite (not infinite and not missing)",
+        command=function() InsertString("is.finite(<variable>)"))
+  tkadd(menu.const.is, "command", label="Infinite",
+        command=function() InsertString("is.infinite(<variable>)"))
+  tkadd(menu.const, "cascade", label="Which elements are ", menu=menu.const.is)
+
   menu.string <- tkmenu(tt, tearoff=0)
   tkadd(top.menu, "cascade", label="String", menu=menu.string, underline=0)
   tkadd(menu.string, "command", label="Concatenate",
@@ -389,54 +487,65 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
           InsertString(paste("substr(<variable>, start = <first element>,",
                              "stop = <last element>)"))
         })
-  
-  menu.tool <- tkmenu(tt, tearoff=0)
-  tkadd(top.menu, "cascade", label="Tool", menu=menu.tool, underline=0)
-  tkadd(menu.tool, "command", label="Build format for date-time object\u2026",
+
+  menu.tools <- tkmenu(tt, tearoff=0)
+  tkadd(top.menu, "cascade", label="Tools", menu=menu.tools, underline=0)
+  tkadd(menu.tools, "command", label="Build format for date-time variable\u2026",
         command=CallFormatDateTime)
 
   # Finalize top menu
-
   tkconfigure(tt, menu=top.menu)
 
   # Frame 0, ok and cancel buttons, and size grip
 
   frame0 <- tkframe(tt, relief="flat", padx=0, pady=0)
 
-  frame0.but.2 <- ttkbutton(frame0, width=12, text="OK",
+  frame0a <- tkframe(frame0, relief="flat", padx=0, pady=0)
+  frame0a.chk.1 <- ttkcheckbutton(frame0a, text="Inverse", variable=inverse.var)
+  frame0a.chk.2 <- ttkcheckbutton(frame0a, text="Hyperbolic",
+                                  variable=hyperbolic.var)
+  frame0a.rad.3 <- ttkradiobutton(frame0a, variable=angles.var, value="rad",
+                                  text="Radians")
+  frame0a.rad.4 <- ttkradiobutton(frame0a, variable=angles.var, value="deg",
+                                  text="Degrees")
+  tkgrid(frame0a.chk.1, frame0a.chk.2, frame0a.rad.3, frame0a.rad.4)
+  tkgrid.configure(frame0a.chk.2, padx=c(2, 12))
+  tkgrid.configure(frame0a.rad.3, padx=c(0, 2))
+
+  frame0.but.3 <- ttkbutton(frame0, width=12, text="OK",
                             command=SaveFunction)
-  frame0.but.3 <- ttkbutton(frame0, width=12, text="Cancel",
+  frame0.but.4 <- ttkbutton(frame0, width=12, text="Cancel",
                             command=function() tclvalue(tt.done.var) <- 1)
-  frame0.but.4 <- ttkbutton(frame0, width=12, text="Help",
+  frame0.but.5 <- ttkbutton(frame0, width=12, text="Help",
                             command=function() {
                               print(help("EditFunction", package="RSurvey"))
-                            }) 
-  frame0.grp.5 <- ttksizegrip(frame0)
+                            })
+  frame0.grp.6 <- ttksizegrip(frame0)
 
-  tkgrid("x", frame0.but.2, frame0.but.3, frame0.but.4, frame0.grp.5)
+  tkgrid(frame0a, "x", frame0.but.3, frame0.but.4, frame0.but.5,
+         frame0.grp.6)
 
-  tkgrid.columnconfigure(frame0, 0, weight=1)
+  tkgrid.columnconfigure(frame0, 1, weight=1)
 
-  tkgrid.configure(frame0.but.2, frame0.but.3, frame0.but.4, 
+  tkgrid.configure(frame0a, padx=10, pady=c(0, 10), sticky="sw")
+  tkgrid.configure(frame0.but.3, frame0.but.4, frame0.but.5,
                    padx=c(0, 4), pady=c(15, 10))
-  tkgrid.configure(frame0.but.4, columnspan=2, padx=c(0, 10))
+  tkgrid.configure(frame0.but.5, columnspan=2, padx=c(0, 10))
+  tkgrid.configure(frame0.grp.6, sticky="se")
 
-  tkgrid.configure(frame0.grp.5, sticky="se")
-
-  tkraise(frame0.but.4, frame0.grp.5)
+  tkraise(frame0.but.5, frame0.grp.6)
 
   tkpack(frame0, fill="x", side="bottom", anchor="e")
 
   # Paned window
-
   pw <- ttkpanedwindow(tt, orient="horizontal")
 
   # Frame 1
 
   frame1 <- tkframe(pw, relief="flat", padx=0, pady=0)
 
-  frame1.lab.1.1 <- ttklabel(frame1, text="Double click to insert variable",
-                             foreground="#414042")
+  txt <- "Double click to insert variable"
+  frame1.lab.1.1 <- ttklabel(frame1, text=txt, foreground="#141414")
   frame1.lst.2.1 <- tklistbox(frame1, selectmode="browse", activestyle="none",
                               relief="flat", borderwidth=5, width=25, height=8,
                               exportselection=FALSE, listvariable=variable.var,
@@ -450,7 +559,7 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
               yscrollcommand=paste(.Tk.ID(frame1.ysc.2.2), "set"))
   tkconfigure(frame1.ysc.2.2, command=paste(.Tk.ID(frame1.lst.2.1), "yview"))
   tcl(frame1.box.3.1, "current", 0)
-  
+
   frame1.lst.4.1 <- tklistbox(frame1, selectmode="browse", activestyle="none",
                               relief="flat", borderwidth=5, width=25, height=5,
                               exportselection=FALSE, listvariable=value.var,
@@ -462,7 +571,7 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
               yscrollcommand=paste(.Tk.ID(frame1.ysc.4.2), "set"))
   tkconfigure(frame1.ysc.4.2, command=paste(.Tk.ID(frame1.lst.4.1), "yview"))
   tkconfigure(frame1.but.5.1, state="disabled")
-  
+
   tkgrid(frame1.lab.1.1, "x")
   tkgrid(frame1.lst.2.1, frame1.ysc.2.2)
   tkgrid(frame1.box.3.1, "x")
@@ -487,14 +596,26 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
 
   txt <- "Define function"
   if (!is.null(index) && edit.fun.id != "")
-    txt <- paste(txt, " for \"", edit.fun.id, "\"", sep="")
-  frame2.lab.1.1 <- ttklabel(frame2, text=txt, foreground="#414042")
-  
-  fnt <- tkfont.create(family="Courier New", size=9)
-  frame2.txt.2.1 <- tktext(frame2, bg="white", font=fnt, padx=2, pady=2,
-                       width=50, height=12, undo=1, wrap="none",
-                       foreground="black", relief="flat",
-                       yscrollcommand=function(...) tkset(frame2.ysc.2.2,...))
+    txt <- paste0(txt, " for \"", edit.fun.id, "\"")
+  frame2.lab.1.1 <- ttklabel(frame2, text=txt, foreground="#141414")
+
+  if (is.null(value.length)) {
+    txt <- ""
+  } else {
+    txt <- paste("(resulting vector must be of length", format(value.length,
+                                                               big.mark=","))
+    if (!is.null(value.class))
+      txt <- paste(txt, "and class", value.class)
+    txt <- paste0(txt, ")")
+  }
+  frame2.lab.1.2 <- ttklabel(frame2, text=txt, foreground="#A40802")
+
+  frame2.txt.2.1 <- tktext(frame2, bg="white", font="TkFixedFont",
+                           padx=2, pady=2, width=50, height=12, undo=1,
+                           autoseparators=1, wrap="none", foreground="black",
+                           relief="flat",
+                           yscrollcommand=function(...)
+                                            tkset(frame2.ysc.2.2, ...))
 
   frame2.ysc.2.2 <- ttkscrollbar(frame2, orient="vertical")
   tkconfigure(frame2.ysc.2.2, command=paste(.Tk.ID(frame2.txt.2.1), "yview"))
@@ -520,31 +641,43 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
                               command=function() InsertString(" == "))
   frame2a.but.10 <- ttkbutton(frame2a, width=3, text="\u2260",
                               command=function() InsertString(" != "))
-  frame2a.but.11 <- ttkbutton(frame2a, width=3, text="( )",
+  frame2a.but.11 <- ttkbutton(frame2a, width=3, text="in",
+                              command=function() InsertString(" %in% "))
+  frame2a.but.12 <- ttkbutton(frame2a, width=3, text="( )",
                               command=function() InsertString("()"))
-  frame2a.but.12 <- ttkbutton(frame2a, width=3, text="[ ]",
+  frame2a.but.13 <- ttkbutton(frame2a, width=3, text="[ ]",
                               command=function() InsertString("[]"))
-  
+  frame2a.but.14 <- ttkbutton(frame2a, width=3, text="\u25C0\u2212",
+                              command=function() InsertString(" <- "))
+  frame2a.but.15 <- ttkbutton(frame2a, width=3, text="\u0078\u207F",
+                              command=function() InsertString("^"))
+
   tkgrid(frame2a.but.01, frame2a.but.02, frame2a.but.03, frame2a.but.04,
-         frame2a.but.05, frame2a.but.06, frame2a.but.07, frame2a.but.08, 
-         frame2a.but.09, frame2a.but.10, frame2a.but.11, frame2a.but.12, 
-         padx=c(0, 2), pady=c(4, 0))
+         frame2a.but.05, frame2a.but.06, frame2a.but.07, frame2a.but.08,
+         frame2a.but.09, frame2a.but.10, frame2a.but.11, frame2a.but.12,
+         frame2a.but.13, frame2a.but.14, frame2a.but.15, pady=c(4, 0))
 
-  tkgrid.configure(frame2a.but.01, padx=c(2, 2))
-  tkgrid.configure(frame2a.but.05, frame2a.but.11, padx=c(12, 2))
-  tkgrid.configure(frame2a.but.12, padx=c(2, 0))
+  tkgrid.configure(frame2a.but.01, frame2a.but.02, frame2a.but.03,
+                   frame2a.but.04, frame2a.but.06, frame2a.but.07,
+                   frame2a.but.08, frame2a.but.09, frame2a.but.10,
+                   frame2a.but.11, frame2a.but.13, frame2a.but.14,
+                   frame2a.but.15, padx=c(2, 0))
+  tkgrid.configure(frame2a.but.05, frame2a.but.12, padx=c(12, 0))
 
-  tkgrid(frame2.lab.1.1, "x")
-  tkgrid(frame2.txt.2.1, frame2.ysc.2.2)
-  tkgrid(frame2a, "x")
-  
-  tkgrid.configure(frame2.lab.1.1, padx=c(2, 0),  pady=c(10, 0), sticky="w")
-  tkgrid.configure(frame2.txt.2.1, padx=c(2, 0),  pady=c(2, 0),  sticky="nsew")
-  tkgrid.configure(frame2.ysc.2.2, padx=c(0, 10), pady=c(2, 0),  sticky="ns")
-  tkgrid.configure(frame2a, pady=c(0, 0), sticky="we")
+  tkgrid(frame2.lab.1.1, frame2.lab.1.2, "x")
+  tkgrid(frame2.txt.2.1, "x", frame2.ysc.2.2)
+  tkgrid(frame2a, "x", "x")
+
+  tkgrid.configure(frame2.lab.1.1, frame2.lab.1.2, padx=c(2, 0), pady=c(10, 0),
+                   sticky="w")
+  tkgrid.configure(frame2.txt.2.1, padx=c(2, 0),  pady=c(2, 0), columnspan=2,
+                   sticky="nsew")
+  tkgrid.configure(frame2.ysc.2.2, padx=c(0, 10), pady=c(2, 0), columnspan=2,
+                   sticky="ns")
+  tkgrid.configure(frame2a, pady=0, columnspan=2, sticky="we")
 
   tkgrid.rowconfigure(frame2, 1, weight=1)
-  tkgrid.columnconfigure(frame2, 0, weight=1, minsize=20)
+  tkgrid.columnconfigure(frame2, 1, weight=1, minsize=20)
 
   if (!is.null(old.fun) && old.fun != "")
     tkinsert(frame2.txt.2.1, "end", old.fun)
@@ -552,9 +685,8 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
   tcl(frame2.txt.2.1, "edit", "reset")
 
   tkmark.set(frame2.txt.2.1, "insert", "end")
-  
-  # Pack frames into paned window
 
+  # Pack frames into paned window
   tkadd(pw, frame1, weight=0)
   tkadd(pw, frame2, weight=1)
   tkpack(pw, fill="both", expand="yes")
@@ -564,7 +696,7 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
   tclServiceMode(TRUE)
 
   tkbind(tt, "<Destroy>", function() tclvalue(tt.done.var) <- 1)
-  
+
   tkbind(frame1.lst.2.1, "<<ListboxSelect>>", ChangeVar)
   tkbind(frame1.lst.2.1, "<Double-ButtonRelease-1>", InsertVar)
   tkbind(frame1.lst.4.1, "<Double-ButtonRelease-1>", InsertValue)
@@ -574,12 +706,13 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
   tkbind("Text", "<Control-y>", EditRedo)
   tkbind("Text", "<Control-v>", EditPaste)
   tkbind("Text", "<Control-a>", EditSelectAll)
+  tkbind("Text", "<Control-l>", ClearConsole)
 
   # GUI control
 
   tkfocus(frame2.txt.2.1)
-  tkgrab(tt)
 
+  tkgrab(tt)
   tkwait.variable(tt.done.var)
 
   tclServiceMode(FALSE)
@@ -587,5 +720,5 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
   tkdestroy(tt)
   tclServiceMode(TRUE)
 
-  rtn
+  return(rtn)
 }
